/*  Sirikata
 *  scriptable.em
 *
 *  Copyright (c) 2011, Ewen Cheslack-Postava
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of Sirikata nor the names of its contributors may
 *    be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

system.require('std/core/bind.em');
system.require('std/core/deepCopy.em');

if (typeof(std) === "undefined") std = {};
if (typeof(std.script) === "undefined") std.script = {};

(
function() {

    var ns = std.script;

    /** A Scriptable is an object which listens for messages from
     *  other objects, executes their contents, and replies with the
     *  results.
     */
    ns.Scriptable = function(avatar, sim) {
		this.avatar = avatar;
        var scriptRequestPattern = new util.Pattern("request", "script");
        var scriptRequestHandler = std.core.bind(this._handleScriptRequest, this);
        scriptRequestHandler << scriptRequestPattern;
		
		/************************************************************************************************************************************/
		if (this.avatar) {
			var scriptRequestGUIStr = @sirikata.ui(
				'scriptRequestGUI',
				function() {
					$('<div id="scriptRequest-ui" title="Script Request">' +
					  '  <div id="ScriptRequestFor"/>' +			
					  '  <div id="ScriptRequestFrom"/>' +	
					  '  <div id="ScriptRequestScript"/>' +	
					  '  <div id="ScriptRequestNotes"/>' +	
					  '  <button id="accept-button">Yes!</button>' +
					  '  <button id="reject-button">No!</button>' +
					  '  <button id="blockScript-button">Block User!</button>' +
					  '</div>').appendTo('body');

					var window = new sirikata.ui.window(                  
					   "#scriptRequest-ui",
					   {
						  autoOpen: false,
						  width: 300,
						  height: 400
					   }
					);
					

					sirikata.ui.button('#accept-button').click(acceptScript);  
					sirikata.ui.button('#reject-button').click(rejectScript);   
					sirikata.ui.button('#blockScript-button').click(blockScripter); 

					var scriptRequests = [];
					var currentRequest;
					var blockedList = {};
					
					function acceptScript() {
						window.toggle();				
						sirikata.event('response', 'accept', currentRequest);
						if (scriptRequests.length > 0) {
							currentRequest = scriptRequests[0];
							scriptRequests.splice(0,1);
							showScriptRequest(currentRequest);
						}
					}
					function rejectScript() {
						window.toggle();					
						sirikata.event('response', 'reject', currentRequest);
						if (scriptRequests.length > 0) {
							currentRequest = scriptRequests[0];
							scriptRequests.splice(0,1);
							showScriptRequest(currentRequest);
						}
					}
					function blockScripter() {
						window.toggle();					
						blockedList[currentRequest.sender] = true;
						sirikata.event('response', 'reject', currentRequest);
						if (scriptRequests.length > 0) {
							currentRequest = scriptRequests[0];
							scriptRequests.splice(0,1);
							showScriptRequest(currentRequest);
						}
					}
					
					function showScriptRequest(request) {
						window.show();
						if (typeof(request.to) !== 'undefined')
							$( "#ScriptRequestFor" ).text('For: ' +request.to);
						if (typeof(request.script) !== 'undefined')
							$( "#ScriptRequestScript" ).text('Script: ' +request.script);
						if (typeof(request.sender) !== 'undefined')
							$( "#ScriptRequestFrom" ).text('From: ' +request.sender);
						if (typeof(request.notes) !== 'undefined')
							$( "#ScriptRequestNotes" ).text('Comments: ' +request.notes);
					}
					
					addRequest = function(sRequest) {
					  if (sRequest.sender in blockedList)
						return;
					  if (scriptRequests.length === 0) {
						currentRequest = sRequest;
						showScriptRequest(sRequest);
					  } else {
						scriptRequests.push(sRequest);
					  }
					};
					
				}
			);@;
			
			var scriptGUIfunc = function (gui) {
				gui.bind("response", std.core.bind(this._handleScriptAnswer,this));
				this.requestGUI = gui;
				system.__debugPrint('testing! \n\n\n\n\n\n\n');

			};
			
			//system.print(tradeStr + ' is the trade string.');
			system.__debugPrint('ADDING SOME GUI \n\n\n');
			sim.addGUITextModule("scriptRequestGUI", scriptRequestGUIStr, std.core.bind(scriptGUIfunc, this));
		}
		/************************************************************************************************************************************/
    };
	
    ns.Scriptable.prototype._handleScriptAnswer = function(response,script) {
		if (response === 'accept') {
		    var sender = system.createVisible(script.sender);
			var cmd = script.script;
			var result = undefined, excep = undefined;
			try {
				result = system.eval(cmd);
			} catch (ex) {
				// Currently, we need to do a deep copy because some
				// fields seem to be some sort of weird built in type (or
				// accessor) which the C++ V8 API doesn't seem to
				// expose. Copying forces the data to be evaluated into
				// real values. So far we've only seen this with
				// exceptions thrown by the V8 runtime (ones generated by
				// our C++ code are fine)
				excep = std.core.deepCopy(ex);
			}
			var retmsg = {
				reply : 'script',
				value : result,
				exception : excep
			};
			retmsg >> sender >> [];
		}
	};
	
	ns.Scriptable.prototype._sendScriptRequest = function(msg, scripter) {
		
		this.requestGUI.call('addRequest',{script: msg.script, sender: scripter.toString()});
	};
    ns.Scriptable.prototype._handleScriptRequest = function(msg, sender) {
        if (!this._printer || sender != this._printer) {
            this._printer = sender;
            system.onPrint( std.core.bind(this._handlePrint, this) );
        }

        if (this._printerTimeout)
            this._printerTimeout.reset();
        this._printerTimeout = system.timeout(60, std.core.bind(this._handlePrinterTimeout, this));
		if (this.avatar && sender.toString() !== system.self.toString())
			this._sendScriptRequest(msg,sender);
		else {
		
			var cmd = msg.script;
			var result = undefined, excep = undefined;
			try {
				result = system.eval(cmd);
			} catch (ex) {
				// Currently, we need to do a deep copy because some
				// fields seem to be some sort of weird built in type (or
				// accessor) which the C++ V8 API doesn't seem to
				// expose. Copying forces the data to be evaluated into
				// real values. So far we've only seen this with
				// exceptions thrown by the V8 runtime (ones generated by
				// our C++ code are fine)
				excep = std.core.deepCopy(ex);
			}
			var retmsg = {
				reply : 'script',
				value : result,
				exception : excep
			};
			retmsg >> sender >> [];
		}
    };

    ns.Scriptable.prototype._handlePrint = function() {
        
        
        var print_msg = {
            request: 'print',
            print: (arguments.length == 1 ? arguments[0] : arguments)
        };
        print_msg >> this._printer >> [];

    };

    ns.Scriptable.prototype._handlePrinterTimeout = function() {
        system.onPrint(undefined);
        this._timerPrintout = null;
    };

})();
